/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 10.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse;

import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.texteditor.ITextEditor;

import com.mbartl.viimplementation.interfaces.IEditor;
import com.mbartl.viimplementation.util.Region;
import com.mbartl.viplugineclipse.actions.AddOneLevelOfIndentation;
import com.mbartl.viplugineclipse.actions.EraseBackOneWord;
import com.mbartl.viplugineclipse.actions.EraseOneLevelOfIndentation;
import com.mbartl.viplugineclipse.actions.SwitchToCommandMode;
import com.mbartl.viplugineclipse.foldingextension.FoldingExtensionFactory;
import com.mbartl.viplugineclipse.foldingextension.IFoldingExtension;

/**
 * TextModificator is a utility class for all kinds of operations on text
 * strings.
 * 
 * @author Michael Bartl
 */
public class EditorImplementation implements IEditor {
	private final ITextEditor _editor;

	private final ITextViewer _viewer;

	private final IFoldingExtension _foldingExtension;
	
	private Vector _lastSearchHighlightingRegions = new Vector();

	private final static Color GRAY = Display.getDefault().getSystemColor(
			SWT.COLOR_GRAY);

	private final static Color YELLOW = Display.getDefault().getSystemColor(
			SWT.COLOR_YELLOW);

	private final static Color BLACK = Display.getDefault().getSystemColor(
			SWT.COLOR_BLACK);

	/**
	 * Creates initialized instance.
	 * 
	 * @param textEditor
	 *            text editor
	 * @param textViewer
	 *            text viewer
	 */
	public EditorImplementation(ITextEditor editor, ITextViewer viewer) {
		_editor = editor;
		_viewer = viewer;
		_foldingExtension = FoldingExtensionFactory.getFoldingExtension(
				_editor, _viewer);
		// activate actions
		editor.setAction(SwitchToCommandMode.ACTIONID,
				new SwitchToCommandMode());
		editor.setAction(EraseBackOneWord.ACTIONID, new EraseBackOneWord());
		editor.setAction(EraseOneLevelOfIndentation.ACTIONID,
				new EraseOneLevelOfIndentation());
		editor.setAction(AddOneLevelOfIndentation.ACTIONID,
				new AddOneLevelOfIndentation());
	}

	public void setCaretPosition(final int pos) {
		int currentPos = getCaretPosition();
		if (pos != currentPos) {
			int validPos = pos;
			boolean forward = currentPos < pos;
			validPos = _foldingExtension.getValidCaretPosition(pos, forward);
			_viewer.getTextWidget().setCaretOffset(validPos);
			_viewer.getTextWidget().showSelection();
		}
	}

	public int getCaretPosition() {
		return _foldingExtension.cursorOffset2DocumentOffset(_viewer
				.getTextWidget().getCaretOffset());
	}

	public void quit(boolean forceClose) {
		try {
			if (forceClose || !_editor.isSaveOnCloseNeeded()) {
				_editor.close(false);
			} else {
				MessageDialog.openInformation(_editor.getSite()
						.getWorkbenchWindow().getShell(), "Can't close buffer",
						"There are unsaved changes. To override use :q!");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void save() {
		try {
			IProgressMonitor pm = _editor.getEditorSite().getActionBars()
					.getStatusLineManager().getProgressMonitor();
			_editor.doSave(pm);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public int getTopIndex() {
		return _viewer.getTopIndex();
	}

	public int getBottomIndex() {
		return _viewer.getBottomIndex();
	}

	public void copySelectionToClipboard(
			com.mbartl.viimplementation.util.Region region) {
		try {
			// Create a clipboard
			Clipboard clipboard = new Clipboard(Display.getDefault());
			// Get the contents of region
			String contents = _viewer.getDocument().get(region.getOffset(),
					region.getLength());
			// Set the clipboard contents
			try {
				clipboard.setContents(new Object[] { contents },
						new Transfer[] { TextTransfer.getInstance() });
			} catch (SWTError e) {
				// may happen in Eclipse 2.1
			}
			clipboard.dispose();
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}

	public void setSelection(com.mbartl.viimplementation.util.Region region) {
		resetHighlighting(false);
		highlightVisualSelection(region);
	}

	public com.mbartl.viimplementation.util.Region getSelection() {
		ITextSelection sel = (ITextSelection) _editor.getSelectionProvider()
				.getSelection();
		return new com.mbartl.viimplementation.util.Region(sel.getOffset(), sel
				.getLength());
	}

	public void revertToSaved() {
		_editor.doRevertToSaved();
	}

	public void srollToLine(int line) {
		_viewer.setTopIndex(line);
	}

	public void highlightSearchRegions(Vector regions) {
		_lastSearchHighlightingRegions = regions;
		for (int i = 0; i < regions.size(); i++) {
			Region region = (Region) regions.get(i);
			region = _foldingExtension.model2viewRegion(region);
			if (region != null) {
				StyleRange style = new StyleRange(region.getOffset(), region
						.getLength(), YELLOW, GRAY);
				_viewer.getTextWidget().setStyleRange(style);
			}
		}
	}

	private void highlightVisualSelection(Region selection) {
		selection = _foldingExtension.model2viewRegion(selection);
		if (selection != null) {
			StyleRange style = new StyleRange(selection.getOffset(), selection
					.getLength(), BLACK, GRAY);
			_viewer.getTextWidget().setStyleRange(style);
		}
	}

	public void resetHighlighting(boolean invalidateSearchHighlighting) {
		if (invalidateSearchHighlighting) {
			_lastSearchHighlightingRegions = new Vector();
		}
		_viewer.invalidateTextPresentation();
		highlightSearchRegions(_lastSearchHighlightingRegions);
	}

	public boolean callAction(String actionId) {
		ViPluginPlugin.getDefault().log("callAction = " + actionId);
		IAction action = null;
		if ("NEW_LINE_BELOW".equals(actionId)) {
			action = _editor.getAction("SmartEnter");
		} else if (actionId.equals("NEW_LINE_ABOVE")) {
			action = _editor.getAction("SmartEnterInverse");
		} else if (actionId.equals("OPEN_DECLARATION")) {
			// not implemented yet
		} else if (actionId.equals("INDENT_LINE")) {
			action = _editor.getAction("Indent");
		}
		if (action != null) {
			action.run();
			return true;
		}
		return false;
	}

	public String getClipboardContent() {
		Clipboard clipboard = new Clipboard(Display.getDefault());
		TextTransfer textTransfer = TextTransfer.getInstance();
		String content = (String) clipboard.getContents(textTransfer);
		clipboard.dispose();
		return content;
	}
}