/*******************************************************************************
 * Copyright (c) 2003-2004, Michael Bartl
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the license in the file License.txt
 * 
 * Created on 28.06.2004
 *******************************************************************************/

package com.mbartl.viplugineclipse.foldingextension;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.ITextViewerExtension5;
import org.eclipse.jface.text.Region;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * The FoldingExtension is used for all TextEditors that utilize the Eclipse
 * folding extension by implementing the ITextViewerExtension5 (since Eclipse
 * 3.0)
 * 
 * @author Michael Bartl
 */
public class FoldingExtension implements IFoldingExtension {
	private ITextEditor _editor;

	private ITextViewer _viewer;

	private ITextViewerExtension5 _extension;

	private IDocument _document;

	/**
	 * Creates initialized instance.
	 * 
	 * @param textEditor
	 *            text editor
	 * @param textViewer
	 *            text viewer
	 */
	public FoldingExtension(ITextEditor editor, ITextViewer viewer)
			throws Exception {
		_editor = editor;
		_viewer = viewer;
		_document = _editor.getDocumentProvider().getDocument(
				_editor.getEditorInput());
		if (_viewer != null && _viewer instanceof ITextViewerExtension5) {
			_extension = (ITextViewerExtension5) _viewer;
		} else
			throw new Exception("No Folding Extension enabled!");
	}

	/**
	 * Gets the next/previous valid caret position.
	 * 
	 * @param pos
	 *            The position to calculate the next/previous postion from
	 * @param forward
	 *            If true the next postion is given, else the previous
	 * @return The valid caret position
	 */
	public int getValidCaretPosition(int pos, boolean forward) {
		int newPos = _extension.modelOffset2WidgetOffset(pos);
		if (newPos != -1) {
			return newPos;
		}
		
		IRegion[] reg = _extension.getCoveredModelRanges(new Region(0,
				_document.getLength()));
		if (forward) {
			for (int i = 0; i < reg.length; i++) {
				if (reg[i].getOffset() > pos) {
					return _extension.modelOffset2WidgetOffset(reg[i]
							.getOffset());
				}
			}
		} else {
			for (int i = reg.length - 1; i >= 0; i--) {
				if (pos >= reg[i].getOffset()) {
					return _extension.modelOffset2WidgetOffset(reg[i]
							.getOffset()
							+ reg[i].getLength() - getDelimiter().length());
				}
			}
		}
		return newPos;
	}

	/**
	 * Get the delimiter of the first line, or a legal delimiter if none is
	 * present.
	 * 
	 * @return The delimiter used in this document or a legal delimiter if none
	 *         is present.
	 */
	private String getDelimiter() {
		String delim = null;
		try {
			delim = _document.getLineDelimiter(0);
		} catch (BadLocationException e) {
			// nothing of interest here
		}
		if (delim == null) {
			delim = _document.getLegalLineDelimiters()[0];
		}

		return delim;
	}

	/**
	 * @return The document position of the given cursor position
	 */
	public int cursorOffset2DocumentOffset(int cursorPosition) {
		return _extension.widgetOffset2ModelOffset(cursorPosition);
	}

	public com.mbartl.viimplementation.util.Region model2viewRegion(
			com.mbartl.viimplementation.util.Region region) {
		IRegion[] reg = _extension.getCoveredModelRanges(new Region(region
				.getOffset(), region.getLength()));
		if (reg != null) {
			return new com.mbartl.viimplementation.util.Region(_extension
					.modelOffset2WidgetOffset(region.getOffset()), region
					.getLength());
		}
		return null;
	}
}